const wpsg_ticket_qrScanner = async (wpsgSettings, data) => {

	const { ref, createApp, onMounted } = await import(`${wpsgSettings.baseUrl}mods/mod_ticket/assets/js/vue.esm-browser.js`);

	createApp({
		setup() {

			let last_scan = undefined;
			let html5QrcodeScanner;

			const ticket_dialog = ref(undefined);

			const xhr = (action, param, callback_ok) => {

				param['action'] = action;

				const request = new XMLHttpRequest();

				request.open('POST', data.admin_url + '?action=wpsg_ticket', true);
				request.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');

				request.onreadystatechange = () => {

					if (request.readyState === 4 && request.status === 200) {

						callback_ok(JSON.parse(request.responseText));

					}

				};

				request.send(JSON.stringify(param));

			};

			const revokeBooking = (ticket_booking_id, hide_dialog) => {

				ticket_dialog.value.loading = true;

				if (hide_dialog === undefined) hide_dialog = true;

				xhr('revokeBooking', {
					booking_id: ticket_booking_id
				}, (response) => {

					if (hide_dialog) hideDialg();
					else {

						ticket_dialog.value = response.ticket_booking;

					}

				});

			};

			const hideDialg = () => {

				html5QrcodeScanner.resume();

				last_scan = undefined;
				ticket_dialog.value = undefined;

			};
			const showTicket = (ticket_booking_id) => {

				ticket_dialog.value.loading = true;

				xhr('getTicketBooking', {
					'ticket_booking_id': ticket_booking_id,
					'log': true
				}, (response) => {

					if (response.found === true) {

						ticket_dialog.value = response;

					} else ticket_dialog.value.loading = false;

				});

			}
			const showDialog = (data) => {

				html5QrcodeScanner.pause();

				ticket_dialog.value = data;

			};
			const en_de_date_time = (value) => {

				const dateObject = new Date(value);

				const day = ('0' + dateObject.getDate()).slice(-2);
				const month = ('0' + (dateObject.getMonth() + 1)).slice(-2);
				const year = dateObject.getFullYear();
				const hours = ('0' + dateObject.getHours()).slice(-2);
				const minutes = ('0' + dateObject.getMinutes()).slice(-2);

    			const germanDateFormat = `${day}.${month}.${year} ${hours}:${minutes}`;

    			return germanDateFormat;

			};

			onMounted(() => {

				document.getElementById('qr_scanner_app').style.display = 'block';

				const onScanSuccess = (decodedText, decodedResult) => {

					if (decodedText === last_scan) return;

					last_scan = decodedText;

					try {

						const arCode = decodedText.split('|');

						if (arCode.length === 3) {

							const order_product_id = parseInt(arCode[0]);
							const amount_index = parseInt(arCode[1]);
							const hash = arCode[2];

							if (order_product_id > 0 && amount_index >= 0) {

								xhr('decodeTicket', {
									'order_product_id': order_product_id,
									'amount_index': amount_index,
									'log': true,
									'hash': hash
								}, (response) => {

									if (response.found === true) {

										showDialog(response);

									} // else last_scan = undefined;

								});

							}

						}

					} catch (error) {

						console.error(error);

					}

				};
				const onScanFailure = (error) => { };

				html5QrcodeScanner = new Html5QrcodeScanner(
				  "reader",
				  { fps: 10, qrbox: { width: 350, height: 350 } },
				  /* verbose */ false);

				html5QrcodeScanner.render(onScanSuccess, onScanFailure);

			});

			return {
				ticket_dialog,
				hideDialg, showDialog, en_de_date_time, revokeBooking, showTicket
			};

		}
	}).mount('#qr_scanner_app');

};

export { wpsg_ticket_qrScanner as wpsg_ticket_qrScanner }