document.addEventListener("DOMContentLoaded", function() {

	for (let el_ticket_product of document.querySelectorAll('.wpsg_ticket_product')) {

		const { createApp, ref, onMounted, watch, computed, nextTick } = Vue;

		const json_data = JSON.parse(atob(el_ticket_product.getElementsByClassName('json_data')[0].innerHTML));

		createApp({
			setup() {

				const data = ref(json_data);
				const mounted = ref(false);
				const ticket = ref(undefined);
				const ref_calendar = ref(undefined);
				const day_selected = ref(undefined);
				const loading = ref(false);
				const layer_msg = ref(undefined);

				const serializeFormToJSON = (formData) => {

					const form_data = {};

					for (let [key, value] of formData.entries()) {

						let parts = key.split(/[\[\]]+/).filter(part => part);

						let currentLevel = form_data;

						for (let i = 0; i < parts.length; i++) {

							let part = parts[i];

							if (i === parts.length - 1) currentLevel[part] = value;
							else {

								if (!currentLevel[part]) currentLevel[part] = {};

								currentLevel = currentLevel[part];

							}

						}

					}

					return form_data;

				};
				const ff = (number) => {

					return parseFloat(number).toFixed(2).replace('.', ',');

				};
				const setVari = (var_id, vari_id) => {

					console.log(var_id, vari_id);

					loading.value = true;

					xhr('setVari', {
						'product_key': data.value.product_key,
						'var_id': var_id,
						'vari_id': vari_id
					}, (response) => {

						data.value.price = response.price;
						data.value.product_key = response.product_key;
						data.value.vp_info = response.vp_info;

						nextTick(() => {

							loading.value = false;

						});

					})

				};
				const xhr = (action, param, callback_ok) => {

					param['action'] = action;
					param['product_id'] = data.value.product_id;

					const request = new XMLHttpRequest();

					request.open('POST', data.value.ajax_url + '?action=wpsg_ticket_frontend', true);
					request.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');

					request.onreadystatechange = () => {

						if (request.readyState === 4 && request.status === 200) {

							callback_ok(JSON.parse(request.responseText));

						}

					};

					request.send(JSON.stringify(param));

				};
				const showDoneLayer = (msg) => {

					layer_msg.value = msg;

				};

				function markEvent(element) {

					element.style.backgroundColor = 'red';
					element.dataset.marked = true;

				}

				onMounted(() => {

					mounted.value = true;

					nextTick(() => {

						const container = ref_calendar.value;
						let shadow = container.attachShadow({ mode: 'open' });

						const appContainer = document.createElement('div');
						shadow.appendChild(appContainer);

						const calendar_css_link = document.getElementById('wpsg_ticket-calendar-css');

						if (calendar_css_link) {

							const link = document.createElement('link');
							link.rel = 'stylesheet';
							link.href = calendar_css_link.getAttribute('href');
							shadow.appendChild(link);

						}

						let event_select_id = 0;

						let calendar = new FullCalendar.Calendar(appContainer, {
							locale: 'de',
							height: 'auto',
							timeZone: 'Europe/Berlin',
							events: data.value.ajax_url + '?action=wpsg_ticket_frontend&ticket_action=getEvents&product_id=' + data.value.product_id,
							initialView: 'dayGridMonth',
							themeSystem: 'standard',
							selectable: true,
							headerToolbar: {
								left: 'prev,next,today',
								center: 'title',
								right: 'dayGridMonth,timeGridWeek,timeGridDay'
							},
							eventContent: function(arg) {

								let span = document.createElement('span'); span.classList.add('wpsg_event'); span.setAttribute('data-event_id', arg.event.id);
								let span2 = document.createElement('span'); span2.classList.add('wpsg_event_free_slots');

								if (arg.event.id === event_select_id) span.classList.add('selected');

								span.innerHTML = arg.event.title; span.setAttribute('title', arg.event.title);
								span2.innerHTML = arg.event.extendedProps['free_slots'] + ' frei';

								span.appendChild(span2);

								return { domNodes: [ span ] }

							},
							eventClick: (arg) => {

								loading.value = true;

								event_select_id = arg.event.id;

								for (let el of shadow.querySelectorAll('.wpsg_event')) {

									if (parseInt(el.getAttribute('data-event_id')) !== parseInt(arg.event.id)) el.classList.remove('selected');
									else el.classList.add('selected');

								}

								xhr('getTicket', {
									ticket_id: arg.event.id
								}, (response) => {

									ticket.value = response.ticket;
									loading.value = false;

								});

							}
						});

						calendar.render();

						if (typeof refreshFsLightbox === 'function') refreshFsLightbox();

					});

				});

				const computedButtonClass = computed(() => { return ''; })
				const computedButtonText = computed(() => { return 'In den Warenkorb'; })

				const form = document.getElementById('wpsg_produktform_' + data.value.template_index);

				form.addEventListener('submit', (event) => {

					event.preventDefault(); event.stopPropagation();

					if (loading.value === true) return;

					loading.value = true;

					const formData = new FormData(el_ticket_product.parentNode);
					const form_data = serializeFormToJSON(formData);

					xhr('addProductToBasket', {
						'ticket': ticket.value,
						'product_id': data.value.product_id,
						'form_data': form_data
					}, (response) => {

						loading.value = false;

						if (response.alert) {
							alert(response.alert);
						}
						if (response.showDoneLayer) {
							showDoneLayer(response.showDoneLayer);
						}
						if (response.redirect) {
							window.location.href = response.redirect;
						}

					});

				});

				return {
					computedButtonText, computedButtonClass,
					setVari, ff,
					data, mounted, day_selected, loading, layer_msg, ticket,
					ref_calendar
				}

			}
		}).mount(el_ticket_product);

	}

});