<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 14.10.23
	 * @time: 07:38
	 */
	
	namespace wpsgTicket;
	
	use \wpsg\M1;
	
	abstract class Action {
		
		public static function wpsg_produkt_edit(array $arParam): void {
			
			$arParam[0]['basket_multiple'] = '1';
			
		}
		
		public static function wpsg_renderProdukt_templateSelect(array &$arParam): void {
			
			$product_id = intval(\wpsg_ShopController::getShop()->getProduktID($arParam[0]['id']));

			if (TicketProduct::isTicketProduct($product_id)) {
			
				$arParam[1] = implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, '..', 'views', 'mods', 'mod_ticket', 'produkttemplate.phtml']);
 
			}
			
		}
		
		public static function cronDaily(): void {
			
			foreach (Ticket::findSerial([], true) as $oTicket) {
			
				$oTicket->createTickets();
			
			}
			
		}
		
		public static function admin_menu(): void {
		 
			\add_menu_page(
            	__('QR Scanner', 'wpsg'),
		        __('QR Scanner', 'wpsg'),
		        'manage_options',
		        'wpsgTicketQrScanner',
		        ['\wpsgTicket\QrScanner', 'dispatch'],
		        'dashicons-id',
		        65
		    );
			
		}
		
		public static function wpsg_admin_emailconf(array $arParam): void {
			
			echo \wpsg_drawEMailConfig(
				'ticketmail',
				__('E-Mail mit den PDF Tickets an den Kunden', 'wpsg'),
				__('Diese Mail wird bei bestimmten Bestellzuständen ausgelöst und stell die Tickets dem Kunden als PDFzu.', 'wpsg'));
			
		}
		
		public static function wpsg_admin_emailconf_save(array $arParam): void {
			
			\wpsg_saveEMailConfig("ticketmail");
			
		}
		
		public static function order_view(array $arParam): void {
			
			list($order_id, &$arSubAction) = $arParam;
			
			$oOrder = \wpsg_order::getInstance(intval($order_id));
			
			foreach ($oOrder->getOrderProducts() as $oOrderProduct) {
				
				if (intval($oOrderProduct->getMeta('ticket_id', false, 0)) > 0) {
					
					$arSubAction['ticket'] = [
						'title' => 'Tickets',
						'content' => \wpsg_ShopController::getShop()->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'order_view.phtml']), false)
					];
					
					break;
					
				}
				
			}
			
		}
		
		public static function wpsg_order_view_row(array $arParam): void {
			
			list(&$p, $i) = $arParam;
			
			if (TicketProduct::isTicketProduct(intval($p['product_id']))) {
			
				$oOrderProduct = \wpsg_order_product::getInstance(intval($p['order_product_id']));
				$oTicket = Ticket::getInstance(intval($oOrderProduct->getMeta('ticket_id')));
				
				$oShop = \wpsg_ShopController::getShop();
				$oShop->view['i'] = $i;
				$oShop->view['p'] = $p;
				$oShop->view['oTicket'] = $oTicket;
				$oShop->view['oOrderProduct'] = $oOrderProduct;
				
				$oShop->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'order_view_row.phtml']));
			
			}
			
		}
		
		public static function wpsg_install(): void {
	 
			global $wpdb;
			
			$charset_collate = $wpdb->get_charset_collate();
			
			$sql = "CREATE TABLE ".$wpdb->prefix."wpsg_tickets (
				id INT NOT NULL AUTO_INCREMENT,
				label VARCHAR(500) DEFAULT '' NOT NULL,
				label_short VARCHAR(500) DEFAULT '' NOT NULL,
				p_id INT NOT NULL,
				from_date DATE,
				from_time TIME,
				to_date DATE,
				to_time TIME,
				slots INT,
				state INT DEFAULT 0,
				autoticket TEXT DEFAULT NULL,
				autoticket_id INT(11) DEFAULT 0 NULL,
				PRIMARY KEY  (id),
				KEY autoticket_id (autoticket_id),
				KEY p_id (p_id)
			) $charset_collate;";
			
			require_once(ABSPATH.'wp-admin/includes/upgrade.php');

			dbDelta($sql);
			
			$sql = "CREATE TABLE ".$wpdb->prefix."wpsg_ticket_templates (
				id INT NOT NULL AUTO_INCREMENT,
				config BLOB,
				template_name VARCHAR(500) NOT NULL DEFAULT '',
				defaulttemplate INT(1) NOT NULL DEFAULT 0,
				deleted INT NOT NULL DEFAULT 0,
				PRIMARY KEY  (id)
			) $charset_collate;";
			
			dbDelta($sql);
			
			$sql = "CREATE TABLE ".$wpdb->prefix."wpsg_ticket_booking (
				id INT NOT NULL AUTO_INCREMENT,
				booking_key VARCHAR(255) NOT NULL,
				order_id INT NOT NULL,
				order_product_id INT NOT NULL,
				product_id INT NOT NULL,
				ticket_id INT NOT NULL,
				amount_index INT NOT NULL,
				revoke_user INT(11) NULL DEFAULT NULL,
				revoke_time DATETIME NULL DEFAULT NULL,
				send_user INT(11) NOT NULL DEFAULT 0,
				send_time DATETIME NULL DEFAULT NULL,
				PRIMARY KEY  (id),
				KEY booking_key (booking_key),
				KEY order_id (order_id),
				KEY order_product_id (order_product_id),
				KEY product_id (product_id),
				KEY ticket_id (ticket_id),
				KEY amount_index (amount_index)
			) $charset_collate;";
			
			dbDelta($sql);
			
			$sql = "CREATE TABLE ".$wpdb->prefix."wpsg_ticket_log (
				id INT NOT NULL AUTO_INCREMENT,
				booking_id INT(11) NOT NULL,
				user_id INT(11) NOT NULL,
				date_time DATETIME NOT NULL,
				log INT(11) NOT NULL,
				PRIMARY KEY  (id),
				KEY booking_id (booking_id)
			) $charset_collate;";
			
			dbDelta($sql);
			
			\wpsg_ShopController::getShop()->checkDefault('wpsg_mod_ticket_orderstate_active', '100,110');
			\wpsg_ShopController::getShop()->checkDefault('wpsg_mod_ticket_orderstate_send', '100');
			\wpsg_ShopController::getShop()->checkDefault('wpsg_ticketmail_betreff', 'Ihre Tickets');
			
		}
		
		public static function wpsg_calculation_saveProduct_after(array $arParam): void {
		
			list($oCalculation, $p, $db_data, $finish_order) = $arParam;
			
			if (TicketProduct::isTicketProduct(intval($p['product_id']))) {
				
				if ($finish_order) {
				
					if (($p['ticket_id']??0) <= 0) throw new \Exception(__('Ticket ID nicht gesetzt!', 'wpsg'));
					
					$oOrderProduct = \wpsg_order_product::getInstance(intval($p['order_product_id']));
					
					$arSecret = [];
					for ($i = 1; $i <= $p['amount']; $i ++) {
						
						$arSecret[] = base64_encode(random_bytes(64));
						
					}
					
					$oOrderProduct->setMeta('ticket_id', intval($p['ticket_id']));
					$oOrderProduct->setMeta('ticket_secrets', json_encode($arSecret));
					
				}
			
			}
		
		}
		
		public static function wpsg_calculation_addProduct(array $arParam): void {
		
			list(&$p, $ses_data) = $arParam;
			
			if ($ses_data !== false) {
				
				if ($ses_data['ticket_id']) $p['data']['ticket_id'] = $ses_data['ticket_id'];
				
			} else {
				
				if (TicketProduct::isTicketProduct(intval($p['data']['product_id']))) {
					
					$p['data']['ticket_id'] = \wpsg_order_product::getInstance(intval($p['data']['order_product_id']))->getMeta('ticket_id');
					
				}
				
			}
			
		}
		
		public static function basket_updateProduktFromSession(array $arParam): void {
			
			$amount_index = $arParam[0];
			$stock = &$arParam[1];
			
			if (intval($_SESSION['wpsg']['basket'][$amount_index]['ticket_id']??0) > 0) {
				
				$oTicket = Ticket::getInstance(intval($_SESSION['wpsg']['basket'][$amount_index]['ticket_id']));
				
				if ($oTicket->getBookingState() !== Ticket::BOOKING_STATE_OPEN) {
				
					$stock = 0;
					\wpsg_ShopController::getShop()->addFrontendError(__('Ticket ist nicht mehr buchbar und wurde aus dem Warenkorb entfernt.', 'wpsg'));
					
					return;
				
				}
				
				if ($stock > $oTicket->getFreeSlotCount()) {
					
					$stock = $oTicket->getFreeSlotCount();
					\wpsg_ShopController::getShop()->addFrontendError(__('Bestand überschreitet die freien Slots des Tickets, Menge wurde angepasst.', 'wpsg'));
					
					return;
					
				}
				
			}
			
		}
		
		public static function wpsg_basket_row(array $arParam): void {
			
			$product_data = $arParam[0];
			
			if (intval($product_data['ticket_id']??0) > 0) {
			
				$oShop = \wpsg_ShopController::getShop();
				$oShop->view['oTicket'] = Ticket::getInstance(intval($product_data['ticket_id']));
				
				$oShop->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'basket_row.phtml']));
			
			}
			
		}
		
		public static function wpsg_overview_row(array $arParam): void {
			
			$product_data = $arParam[0];
			
			if (intval($product_data['ticket_id']??0) > 0) {
				
				$oShop = \wpsg_ShopController::getShop();
				$oShop->view['oTicket'] = Ticket::getInstance(intval($product_data['ticket_id']));
				$oShop->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'basket_row.phtml']));
			
			}
			
		}
		
		public static function wpsg_product_addedit_content(array $arParam): void {
			
			if (($_REQUEST['action']??'add') === 'add') return;
			
			$content = Plugin::getInstance()->render('product_addedit_content.phtml', [
				'product_data' => $arParam[1]
			]);
			
			$arParam[0]['wpsg_mod_ticket'] = [
				'title' => __('Ticketprodukt', 'wpsg'),
				'content' => $content
			];
			
		}
		
		public static function setOrderStatus(array $arParam): void {
			
			list($order_id, $status_id, $inform) = $arParam;
			
			if (in_array(intval($status_id), Plugin::getInstance()->getSendOrderState())) {
			
				Plugin::getInstance()->sendTickets(intval($order_id));
				
			}
			
		}
	
	}