<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 15.10.23
	 * @time: 09:28
	 */
	
	namespace wpsgTicket;
	
	abstract class Ajax {
		
		public static function backend(): void {
			
			$raw_data = file_get_contents('php://input');
			$request = json_decode($raw_data, true);
			
			if (!current_user_can('administrator' )) throw new \Exception("1");
			
			if (!is_array($request)) { $request = $_REQUEST; $request['action'] = $_REQUEST['ticket_action']; }
			
			$actionName = $request['action'].'Action';
			
			if (method_exists(__NAMESPACE__.'\Ajax', $actionName)) {
				
				$out = self::$actionName($request);
				
				header('Content-Type: application/json');
				
				echo json_encode($out); exit;
				
			} else throw new \Exception('2');
		
		}
		
		public static function frontend(): void {
		 
			$raw_data = file_get_contents('php://input');
			$request = json_decode($raw_data, true);
			
			if (!is_array($request)) { $request = $_REQUEST; $request['action'] = $_REQUEST['ticket_action']; }
			
			$actionName = $request['action'].'FrontendAction';
			
			if (method_exists(__NAMESPACE__.'\Ajax', $actionName)) {
				
				$out = self::$actionName($request);
				
				header('Content-Type: application/json');
				echo json_encode($out); exit;
				
			} else throw new \Exception('2');
			
		}
		
		/* - */
		
		private static function setVariFrontendAction(array $request): array {
 
			$product_key = \wpsg_ShopController::getShop()->callMod(
				'wpsg_mod_productvariants',
				'getSimulatedVariKey', [
					$request['product_key'],
					$request['var_id'],
					$request['vari_id']
				]
			);
			
			$oProduct = \wpsg_product::getInstance($request['product_id']);
			$oProduct->setProductKey($product_key);
			
			$r = [
				'price' => $oProduct->getPrice($product_key, \wpsg_ShopController::getShop()->getFrontendTaxview()),
				'product_key' => $product_key,
			];
		
			if (\wpsg_ShopController::getShop()->hasMod('wpsg_mod_productvariants')) {
				
				if (\wpsg_ShopController::getShop()->callMod('wpsg_mod_productvariants', 'isVariantsProductKey', [$product_key])) {
					
					$r['vp_info'] = \wpsg_ShopController::getShop()->callMod('wpsg_mod_productvariants', 'getVariantenInfoArray', [$product_key]);
					
				}
				
			}
			
			return $r;
			
		}
	
		private static function getTicketFrontendAction(array $request): void {
			
			$ticket_id = intval($request['ticket_id']);
			
			header('Content-Type: application/json');
			echo json_encode(['ticket' => Ticket::getInstance($ticket_id)->toJSON()]); exit;
			
		}
		
		private static function getFrontendAction(array $request): void {
			
			$oOrderProduct = \wpsg_order_product::getInstance(intval($request['order_product_id']));
			$oTicketProduct = TicketProduct::getInstance($oOrderProduct->getProductId());
			
			$ticket_id = intval($oOrderProduct->getMeta('ticket_id'));
			$arTicketSecrets = json_decode($oOrderProduct->getMeta('ticket_secrets'));
			
			$oTicket = Ticket::getInstance($ticket_id);
			
			TicketRenderer::render(
				intval($oTicketProduct->getProduct()->getMeta('ticket_template')),
				$oOrderProduct->getId(),
				intval($request['amount_index'])
			);
			
			exit;
			
		}
		
		private static function addProductToBasketFrontendAction(array $request): array {
			
			//\wpsg_ShopController::getShop()->basket->addProduktToSession($request['form_data']['wpsg[product_key]'], $request['ticket']['amount']);
			
			$product_key = $request['form_data']['wpsg']['product_key'];
			
			if (!wpsg_isSizedArray($_SESSION['wpsg']['basket'])) $_SESSION['wpsg']['basket'] = [];
			
			$oTicketProduct = \wpsgTicket\TicketProduct::getInstance(intval(\wpsg_ShopController::getShop()->getProduktID($product_key)));
			$oTicket = \wpsgTicket\Ticket::getInstance($request['ticket']['id']);
			
			$wpsg_afterinsert = intval(\wpsg_ShopController::getShop()->get_option('wpsg_afterinsert'));
			
			if ($request['ticket']['amount'] > $oTicket->getFreeSlotCount()) {
				
				return [
					'success' => false,
					'alert' => __('Es sind nicht genug freie Slots verfügbar.', 'wpsg')
				];
				
			}
			
			$p = [
				'menge' => $request['ticket']['amount'],
				'id' => $product_key,
				'referer' => '',
				'ticket_id' => $oTicket->getId()
			];
			
			if (isset($request['form_data']['wpsg_mod_productvars'])) {
				
				$p['wpsg_mod_productvars'] = [];
				
				foreach ($request['form_data']['wpsg_mod_productvars'] as $pv_id => $pv_value) {
					
					$p['wpsg_mod_productvars'][$pv_id] = wpsg_xss($pv_value);
					
				}
				
			}
			
			$_SESSION['wpsg']['basket'][] = $p;
			
			$r = [
				'success' => true
			];
			
			if ($wpsg_afterinsert === 0) $r['showDoneLayer'] = __('Produkt wurde erfolgreich in den Warenkorb gelegt.', 'wpsg');
			else $r['redirect'] = \wpsg_ShopController::getShop()->getURL(\wpsg_ShopController::URL_BASKET);
			
			return $r;
			
		}
		
		private static function getEventsFrontendAction(array $request): array {
			
			$start = date('Y-m-d H:i:s', strtotime($request['start']));
			$end = date('Y-m-d H:i:s', strtotime($request['end']));
			$product_id = intval($request['product_id']);
			$arTicketID = [];
			
			foreach (Plugin::getDatesBetween($start, $end) as $day) {
				
				if (strtotime($day) < time()) continue;
				
				$arTicket = Ticket::find([
					'product_id' => $product_id,
					'day' => $day
				]);
				
				foreach ($arTicket as $ticket_id) {
					
					$arTicketID[] = $ticket_id;
					
				}
				
			}
			
			$arReturn = [];
			
			foreach (array_unique($arTicketID) as $ticket_id) {
				
				$oTicket = Ticket::getInstance(intval($ticket_id));
				
				if ($oTicket->getFreeSlotCount() <= 0) continue;
				
				if ($oTicket->isSerialTicket()) {
					
					$label_short = $oTicket->getSerialTicket()->getShortLabel();
					if ($label_short !== '') $ticket_title = $label_short;
					else $ticket_title = $oTicket->getSerialTicket()->getLabel();
				}
				else $ticket_title = $oTicket->getProduct()->getProductName();
				
				$arReturn[] = [
					'title' => $ticket_title,
					'id' => $ticket_id,
					'free_slots' => $oTicket->getFreeSlotCount(),
					'start' => $oTicket->getStartDate()->getTimestamp() * 1000,
					'end' => $oTicket->getEndDate()->getTimestamp() * 1000
				];
				
			}
			
			header('Content-Type: application/json');
			echo json_encode($arReturn); exit;
			
		}
		
		/* Backend */
		
		private static function loadOccupancyAction(array $request): array {
			
			$oTicketProduct = TicketProduct::getInstance(intval($request['product_id']));
			
			$date_from = date('Y-m-d', strtotime('-2 Month'));
			$date_to = date('Y-m-d', strtotime('+4 Month'));
			
			$arOccupancy = $oTicketProduct->getOccupancy($date_from, $date_to);
			
			return [
				'arOccupancy' => $arOccupancy
			];
			
		}
		
		private static function resendAction(array $request): array {
			
			$oOrder = \wpsg_order::getInstance(intval($request['order_id']));
			
			Plugin::getInstance()->sendTickets($oOrder->getId());
			
			\wpsg_ShopController::getShop()->addBackendMessage(__('Tickets wurden erneut versendet.'));
			
			\wpsg_ShopController::getShop()->redirect(
				wpsg_admin_url('Order', 'view', ['edit_id' => $oOrder->getId()])
			); exit;
			
		}
		
		private static function exportAction(array $request): array {
			
			if (isset($request['ticket_product_id'])) {
				
				$ticket_product_id = intval($request['ticket_product_id']);
				$oTicketProduct = TicketProduct::getInstance($ticket_product_id);
				$arTicketBooking = $oTicketProduct->getBookingTickets();
				
			} else if (isset($request['order_id'])) {
				
				$oOrder = \wpsg_order::getInstance($request['order_id']);
				
				/** @var TicketBooking[] $arTicketBooking */
				$arTicketBooking = [];
				
				foreach ($oOrder->getOrderProducts() as $oOrderProduct) {
					
					if (intval($oOrderProduct->getMeta('ticket_id', false, 0)) > 0) {
					
						for ($amount_index = 0; $amount_index < $oOrderProduct->getAmount(); $amount_index ++) {
							
							$arTicketBooking[] = TicketBooking::getInstanceByBookingKey(TicketBooking::encodeBookingKey($oOrderProduct->getId(), $amount_index));
							
						}
					
					}
					
				}
				
				
			}
			
			if (sizeof($arTicketBooking) > 0) {
				
				$csvFilePath = tempnam(sys_get_temp_dir(), 'csv');
				$csvFile = fopen($csvFilePath, 'w');
				
				$arHead = [
					'Ticketbuchung',
					'Serientermin',
					'Zeitraum',
					'Produkt',
					'Bestellung',
					'Käufer',
					'Status',
					'Entwertung'
				];
				
				$arProductVars = \wpsg_ShopController::getShop()->db->fetchAssoc("
					SELECT
						PV.*
					FROM
						`".WPSG_TBL_PRODUCTS_VARS."` AS PV
					WHERE
						1
					ORDER BY
						PV.`pos` ASC, PV.`id` ASC
				");
				
				foreach ($arProductVars as $pv) $arHead[] = $pv['name'];
				
				fputcsv($csvFile, $arHead);
				
				foreach ($arTicketBooking as $oTicketBooking) {
					
					$arRow = [
						$oTicketBooking->getLabel(),
						(($oTicketBooking->getTicket()->isSerialTicket())?$oTicketBooking->getTicket()->getSerialTicket()->getLabel():'--'),
						$oTicketBooking->getTicket()->getLabel(),
						$oTicketBooking->getProduct()->getProduct()->getProductName(),
						$oTicketBooking->getOrder()->getNr(),
						$oTicketBooking->getOrder()->getCustomer()->getFirstname().' '.$oTicketBooking->getOrder()->getCustomer()->getName(),
						$oTicketBooking->getStateLabel(),
						$oTicketBooking->getRevokeTooltip()
					];
					
					$arProductVarData = \wpsg_ShopController::getShop()->callMod('wpsg_mod_productvars', 'getAllProductVarValues', [$oTicketBooking->getOrderID(), $oTicketBooking->getOrderProduct()->getProductIndex()]);
					
					foreach ($arProductVars as $pv) {
						
						$pv_active = \wpsg_ShopController::getShop()->callMod('wpsg_mod_productvars', 'hasProductProductVarActive', [$oTicketBooking->getProductID(), $pv['id']]);
						
						if ($pv_active === true) {
							
							$arRow[] = $arProductVarData[$pv['id']]['value']??'';
							
						} else {
							
							$arRow[] = '';
							
						}
						
					}
					
					fputcsv($csvFile, $arRow);
					
				}
				
				fclose($csvFile);
				
				header('Content-Type: text/csv');
				header('Content-Disposition: attachment; filename="export.csv"');
				
				readfile($csvFilePath);
				unlink($csvFilePath);
				
				exit;
				
			} else {
				
				die("Keine Daten zu exportieren");
				
			}
			
		}
	
		private static function deleteTicketAction(array $request): array {
			
			$oTicket = Ticket::getInstance($request['ticket_id']);
			$oTicket->delete();
			
			return [
				'ticket' => $oTicket->toJSON()
			];
			
		}
		
		private static function getTicketBookingAction(array $request): array {
			
			try {
				
				$oTicketBooking = TicketBooking::getInstance(intval($request['ticket_booking_id']));
				
				return [
					'found' => true,
					'loading' => false,
					'tab' => 1,
				] + $oTicketBooking->toJSON();
				
				throw new \Exception(__('TicketBuchung konnte nicht geladen werden.', 'wpsg'));
				
			} catch (\Exception $e) {
				
				return [
					'found' => false,
					'exception' => $e->getMessage()
				];
				
			}
			
		}
		
		private static function save_ticketsAction(array $request): array {
			
			global $wpdb;
			
			$serial_id = intval($request['serial']['id']??0);
			
			if ($serial_id <= 0) {
				
				$wpdb->show_errors = true;
				
				$wpdb->insert(WPSG_TBL_TICKETS, [
					'autoticket' => json_encode($request['serial']),
					'p_id' => $request['product_id'],
					'label' => $request['serial']['label'],
					'label_short' => $request['serial']['label_short'],
					'from_date' => $request['serial']['from_date'],
					'from_time' => $request['serial']['event_data']['to_time'],
					'to_date' => $request['serial']['to_date'],
					'to_time' => $request['serial']['event_data']['to_time'],
					'slots' =>  $request['serial']['event_data']['slots'],
					'state' => Ticket::STATE_ACTIVE
				], [
					'%s', '%d', '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%d'
				]);
		 
				$ticket_id = $wpdb->insert_id;
				
			} else {
				
				$wpdb->update(WPSG_TBL_TICKETS, [
					'autoticket' => json_encode($request['serial']),
					'p_id' => $request['product_id'],
					'label' => $request['serial']['label'],
					'label_short' => $request['serial']['label_short'],
					'from_date' => $request['serial']['from_date'],
					'from_time' => $request['serial']['event_data']['from_time'],  // Annahme korrekte Zuweisung wie im Insert-Teil
					'to_date' => $request['serial']['to_date'],
					'to_time' => $request['serial']['event_data']['to_time'],
					'slots' => 20, //$request['serial']['event_data']['slots'],
					'state' => Ticket::STATE_ACTIVE
				], [
					'id' => $serial_id
				], [
					'%s', '%d', '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%d'
				], [
					'%d'
				]);

				$ticket_id = $serial_id;
				
			}
			
			$oTicket = Ticket::getInstance($ticket_id);
			$oTicket->createTickets();
			
			return [true];
			
		}
		
		private static function add_ticketsAction(array $request): array {
		
			$oTicketProduct = TicketProduct::getInstance($request['product_id']);
			$oTicketProduct->addTickets(
				$request['data']['from_date'], $request['data']['from_time'],
				$request['data']['to_date'], $request['data']['to_time'],
				$request['data']['slots']
			);
			
			return [true];
			
		}
		
		private static function revokeBookingAction(array $request): array {
			
			$oTicketBooking = TicketBooking::getInstance($request['booking_id']);
			$oTicketBooking->revoke();
			
			return [
				'result' => true,
				'ticket_booking' =>  [
					'found' => true,
					'loading' => false,
					'tab' => 1,
				] + TicketBooking::getInstance($oTicketBooking->getId(), true)->toJSON(true)
			];
			
		}
	
		private static function decodeTicketAction(array $request): array {
			
			try {
				
				$order_product_id = intval($request['order_product_id']);
				$amount_index = intval($request['amount_index']);
				$hash = $request['hash'];
				
				$oOrderProduct = \wpsg_order_product::getInstance($order_product_id);
				$ticket_secrets = $oOrderProduct->getMeta('ticket_secrets');
				
				if (md5($ticket_secrets[$amount_index]) === $hash) {
					
					$oTicketBooking = TicketBooking::getInstanceByBookingKey(
						TicketBooking::encodeBookingKey($order_product_id, $amount_index)
					);
					
					if ($request['log'] === true) {
						
						$oTicketBooking->addBookingLog(TicketBooking::LOG_SCAN);
						
					}
					
					return [
						'found' => true,
						'loading' => false,
						'tab' => 1,
					] + $oTicketBooking->toJSON();
					
				}
				
			} catch (\Exception $e) {
				
				return [
					'found' => false,
					'exception' => $e->getMessage()
				];
				
			}
			
			return [
				'found' => false
			];
			
		}
		
		private static function previewAction(array $request): array {
		
			TicketRenderer::render(intval($request['template_id'])); exit;
			
			return [];
			
		}
		
		private static function saveSettingsAction(array $request): array {
			
			global $wpdb;
			
			\wpsg_ShopController::getShop()->update_option('wpsg_mod_ticket_orderstate_active', implode(',', $request['wpsg_mod_ticket_orderstate_active']??''));
			\wpsg_ShopController::getShop()->update_option('wpsg_mod_ticket_orderstate_send', implode(',', $request['wpsg_mod_ticket_orderstate_send']??''));
			
			foreach ($request['arTemplates'] as $template_data) {
			
				TicketTemplate::save($template_data);
			
			}
			
			return [];
			
		}
		
		private static function getBackgroundAction(array $request): array {
			
			$oTemplate = TicketTemplate::getInstance(intval($request['template_id']));
			
			$file = $oTemplate->getBgFilePath();
			
			$mimeType = mime_content_type($file);

			header("Content-Type: $mimeType");
			header("Content-Length: ".filesize($file));
			header("Content-Disposition: inline; filename=\"".basename($file)."\"");
		
			readfile($file);  // Datei ausgeben
			exit;
			
			return [];
			
		}
		
		private static function uploadBgAction(array $request): array {
			
			TicketTemplate::save($request['template']);
			$oTemplate = TicketTemplate::getInstance($request['template']['id']);
			
			$path = $oTemplate->getStoragePath();
			
			if (file_exists($path)) wpsg_clearDirectory($path); \wpsg_ShopController::getShop()->protectDirectory($path);
			
			file_put_contents($path.$request['file']['fileName'], base64_decode($request['file']['fileData']));
			
			return [
				'template' => $oTemplate->toJSON()
			];
			
		}
		
		private static function addTemplateAction(array $request): array {
			
			global $wpdb;
			
			$wpdb->insert(WPSG_TBL_TICKET_TEMPLATES, [
				'config' => json_encode([
					'width' => 85.6,
					'height' => 53.98,
					'arField' => [
						[
							'x' => 10, 'y' => 10, 'w' => 33.98, 'h' => 33.98,
							'align' => 'L', 'color' => '000000', 'opaque' => '1', 'angle' => 0,
							'text' => '%qr%', 'font' => 'helvetica.php', 'fontsize' => 12
						]
					]
				]),
				'template_name' => $request['template_name']
			], [
				'%s',
				'%s'
			]);
			
			$template_id = $wpdb->insert_id;
			
			return [
				'template' => TicketTemplate::getInstance($template_id)->toJSON()
			];
			
		}
		
		private static function changeStateAction(array $request): array {
			
			$oTicket = Ticket::getInstance($request['ticket_id']);
			$oTicket->changeState();
			
			return [
				'ticket' => $oTicket->toJSON()
			];
			
		}
		
		private static function getTicketAction(array $request): array {
			
			$arPager = $request['pager'];
			
			$arPager['count'] = Ticket::count($request['filter']);
			$arPager['pages'] = ceil($arPager['count'] / $arPager['per_page']);
			$arPager['page'] = max(min(($arPager['page']??1), $arPager['pages']), 1);
			
			$arPagerSerial = $request['pager_serial'];
			
			$arPagerSerial['count'] = Ticket::countSerial($request['filter_serial']);
			$arPagerSerial['pages'] = ceil($arPagerSerial['count'] / $arPagerSerial['per_page']);
			$arPagerSerial['page'] = max(min(($arPagerSerial['page']??1), $arPagerSerial['pages']), 1);
			
			return [
				'items' => Ticket::find($request['filter'], $arPager, false, true),
				'items_serial' => Ticket::findSerial($request['filter_serial'], false, true),
				'pager' => $arPager,
				'pager_serial' => $arPagerSerial
			];
			
		}
		
	}