<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 04.11.23
	 * @time: 11:46
	 */
	
	namespace wpsgTicket;
	
	class PDF extends \setasign\Fpdi\Fpdi {
	
		var $extgstates = [];
		
		function TextWithRotation($x, $y, $txt, $txt_angle, $font_angle = 0): void {
			
		    $txt = str_replace(')', '\\)', str_replace('(', '\\(', str_replace('\\', '\\\\', $txt)));
		
		    $font_angle += 90+$txt_angle;
		    $txt_angle *= M_PI/180;
		    $font_angle *= M_PI/180;
		
		    $txt_dx = cos($txt_angle);
		    $txt_dy = sin($txt_angle);
		    $font_dx = cos($font_angle);
		    $font_dy = sin($font_angle);
		
		    $s = sprintf('BT %.2f %.2f %.2f %.2f %.2f %.2f Tm (%s) Tj ET',
		             $txt_dx, $txt_dy, $font_dx, $font_dy,
		             $x*$this->k, ($this->h-$y)*$this->k, $txt);
			
		    if ($this->ColorFlag)
		        $s='q '.$this->TextColor.' '.$s.' Q';
		    $this->_out($s);
		}
		 
	    function SetAlpha($alpha, $bm='Normal')
	    {
	        // set alpha for stroking (CA) and non-stroking (ca) operations
	        $gs = $this->AddExtGState(array('ca'=>$alpha, 'CA'=>$alpha, 'BM'=>'/'.$bm));
	        $this->SetExtGState($gs);
	    }
	
	    function AddExtGState($parms)
	    {
	        $n = count($this->extgstates)+1;
	        $this->extgstates[$n]['parms'] = $parms;
	        return $n;
	    }
	 	  
	    function SetExtGState($gs)
	    {
	        $this->_out(sprintf('/GS%d gs', $gs));
	    }
		
		function _enddoc()
	    {
	        if(!empty($this->extgstates) && $this->PDFVersion<'1.4')
	            $this->PDFVersion='1.4';
	        parent::_enddoc();
	    }
	
	    function _putextgstates()
	    {
	        for ($i = 1; $i <= count($this->extgstates); $i++)
	        {
	            $this->_newobj();
	            $this->extgstates[$i]['n'] = $this->n;
	            $this->_out('<</Type /ExtGState');
	            foreach ($this->extgstates[$i]['parms'] as $k=>$v)
	                $this->_out('/'.$k.' '.$v);
	            $this->_out('>>');
	            $this->_out('endobj');
	        }
	    }

	    function _putresourcedict()
	    {
	        parent::_putresourcedict();
	        $this->_out('/ExtGState <<');
	        foreach((array)$this->extgstates as $k=>$extgstate)
	            $this->_out('/GS'.$k.' '.$extgstate['n'].' 0 R');
	        $this->_out('>>');
	    }
	
	    function _putresources()
	    {
	        $this->_putextgstates();
	        parent::_putresources();
	    }
	
	}