<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 14.10.23
	 * @time: 07:51
	 */
	
	namespace wpsgTicket;
	
	class Plugin {
		
		protected static $instance = null;
		
		public function __construct() {
		
		}
		
		/**
		 * Erstellt den QR Code
		 *
		 * @param int $order_product_id
		 * @param int $amount_index
		 * @return string
		 */
		public function buildCode(int $order_product_id, int $amount_index): string {
			
			$oOrderProduct = \wpsg_order_product::getInstance($order_product_id);
			$ticket_secrets = $oOrderProduct->getMeta('ticket_secrets');
			
			return $oOrderProduct->getId().'|'.$amount_index.'|'.md5($ticket_secrets[$amount_index]);
			
		}
		
		public function getActiveOrderState(): array {
		
			$wpsg_mod_ticket_orderstate_active = \wpsg_ShopController::getShop()->get_option('wpsg_mod_ticket_orderstate_active');
			if ($wpsg_mod_ticket_orderstate_active === false) return [];
			
			return array_map('intval', explode(',', $wpsg_mod_ticket_orderstate_active));
		
		}
		
		public function getSendOrderState(): array {
		
			$wpsg_mod_ticket_orderstate_send = \wpsg_ShopController::getShop()->get_option('wpsg_mod_ticket_orderstate_send');
			if ($wpsg_mod_ticket_orderstate_send === false) return [];
			
			return array_map('intval', explode(',', $wpsg_mod_ticket_orderstate_send));
		
		}
		
		public function render(string $template, array $view): string {
			
			return \wpsg_ShopController::getShop()->render(WPSG_PATH_VIEW.'mods/mod_ticket/'.$template, false);
			
		}
		
		public function templatePath(): string {
			
			return dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR;
			
		}
		
		public function sendTickets(int $order_id, bool $resend = false): void {
		
			$oOrder = \wpsg_order::getInstance(intval($order_id));
			
			$arTicketBookingIDSend = [];
			
			foreach ($oOrder->getOrderProducts() as $oOrderProduct) {
		
				$oOrderProduct->getMeta('ticket_id', false, 0);
				
				$ticket_id = intval($oOrderProduct->getMeta('ticket_id', false, 0));
				
				if ($ticket_id > 0) {
					
					for ($amount_index = 0; $amount_index < $oOrderProduct->getAmount(); $amount_index ++) {
						
						$oTicketBooking = TicketBooking::getInstanceByBookingKey(TicketBooking::encodeBookingKey($oOrderProduct->getId(), $amount_index));
					
						if ($resend || !$oTicketBooking->isSend()) $arTicketBookingIDSend[] = $oTicketBooking->getId();
						
					}
					
				}
				
			}
			
			if (sizeof($arTicketBookingIDSend) > 0) {
				
				$oShop = \wpsg_ShopController::getShop();
				$oShop->view['arTicketBookingIDs'] = $arTicketBookingIDSend;
				$oShop->view['oOrder'] = $oOrder;
				
				$mail_txt = $oShop->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'ticket_mail.phtml']), false);
				$mail_html = $oShop->render(implode(DIRECTORY_SEPARATOR, [WPSG_PATH_VIEW, 'mods', 'mod_ticket', 'ticket_mail_html.phtml']), false);
				
				$arAttachment = [];
				
				foreach ($arTicketBookingIDSend as $ticket_booking_id) {
					
					$oTicketBooking = TicketBooking::getInstance($ticket_booking_id);
					
					TicketRenderer::render(null, $oTicketBooking->getOrderProductID(), $oTicketBooking->getAmountIndex(), false);
					$arAttachment[] = $oTicketBooking->getTicketPDFPath();
					
				}
				
				$oShop->sendMail($mail_txt, $oOrder->getCustomer()->getEMail(), 'ticketmail', $arAttachment, $oOrder->getId(), $oOrder->getCustomer()->getId(), $mail_html);
				
				foreach ($arTicketBookingIDSend as $ticket_booking_id) {
					
					$oTicketBooking = TicketBooking::getInstance($ticket_booking_id);
					$oTicketBooking->setSend();
					
				}
			
			}
		
		}
		
		/** static */
		
		public static function getInstance(): Plugin {
			
			if (!isset(static::$instance)) static::$instance = new static;
			
			return static::$instance;
			
		}
		
		/**
		 * Gibt einen Array mit Tagen im Format YYYY-MM-DD zurück die zwischen $startDate und $endDate liegen
		 *
		 * @param string $startDate
		 * @param string $endDate
		 * @return string[]
		 * @throws \Exception
		 */
		public static function getDatesBetween(string $startDate, string $endDate): array {
   
			$start = new \DateTime($startDate);
			$end = new \DateTime($endDate);
			$interval = new \DateInterval('P1D');
			$period = new \DatePeriod($start, $interval, $end->modify('+1 day'));

			$arDates = [];
		
			foreach ($period as $date) $arDates[] = $date->format('Y-m-d');
			
			return $arDates;

		}
		
	}