<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 20.10.23
	 * @time: 14:33
	 */
	
	namespace wpsgTicket;
	
	class TicketProduct {
	
		public static bool $enqueTicket = false;
		
		private static array $arInstances = array();
		
		private ?int $product_id = null;
		
		public function __construct(int $product_id) {
		
			$this->product_id = $product_id;
			
		}
		
		public function getProductId(): int { return $this->product_id; }
		public function getProduct(): \wpsg_product { return \wpsg_product::getInstance($this->getProductId()); }
		
		/**
		 * Gibt einen Array von Tagen zwischen $from und $to zurück, an dem Tickets angelegt sind
		 * und zählt auch die Belegung
		 *
		 * @param string $from YYYY-MM-DD
		 * @param string $to YYYY-MM-DD
		 * @return array [YYYY-MM-DD] => $nAnzahlBelegungen
		 * @throws \Exception
		 */
		public function getOccupancy(string $from, string $to): array {
		
			$arReturn = [
				'min' => null,
				'max' => null,
			];
			
			if (!preg_match('/^\d{4}-\d{2}\-\d{2}$/', $from)) throw new \Exception();
			if (!preg_match('/^\d{4}-\d{2}\-\d{2}$/', $to)) throw new \Exception();
			
			$tFrom = strtotime($from); $tTo = strtotime($to);
			
			if ($tFrom > $tTo) throw new \Exception();
			
			$t = $tFrom;
			
			while ($t < $tTo) {
				
				$date = date('Y-m-d', $t);
				
				$arTicket = Ticket::find([
					'day' => $date,
					'product_id' => $this->getProductId()
				], null, true, false);
				
				if (sizeof($arTicket) > 0) {
					
					if ($arReturn['max'] === null || strtotime($arReturn['max']) < $t) $arReturn['max'] = $date;
					if ($arReturn['min'] === null || strtotime($arReturn['min']) > $t) $arReturn['min'] = $date;
					
					foreach ($arTicket as $oTicket) {
						
						if (!isset($arReturn[$date])) {
							
							$arReturn[$date] = [
								'arTicket' => [],
								'slots' => 0,
								'slots_used' => 0
							];
							
						}
						
						$arReturn[$date]['arTicket'][] = $oTicket->toJSON();
						$arReturn[$date]['slots'] += $oTicket->getSlotCount();
						$arReturn[$date]['slots_used'] += $oTicket->getUsedSlotCount();
						
					}
					
				}
				
				$t = strtotime('+1 day', $t);
				
			}
			
			return $arReturn;
			
			$results = $wpdb->get_results($wpdb->prepare("
				SELECT
					CAST('%s' AS `date`) AS `get_from`,
					CAST('%s' AS `date`) AS `get_to`,
				FROM
					`".WPSG_TBL_TICKETS."` AS T
				WHERE
					T.`p_id` = %d
				HAVING
					(T.`from_date` >= `get_from` AND T.`from_date` <= `get_to` AND T.`to_date` IS NULL) OR
					(T.`from_date`T.`to_date` >= `get_from` )
			"), $from, $to, $this->getProductId());
		
		}
		
		public function getBookingTickets(): array {
			
			global $wpdb;
			
			$arReturn = [];
			
			foreach (Ticket::find([
				'product_id' => $this->getProductId(),
				'state' => -1
			], null, true) as $oTicket) {
			 
				$results = $wpdb->get_results($wpdb->prepare("
					SELECT
					    OP.`id` AS `order_product_id`, M.`meta_value`
					FROM
						`".WPSG_TBL_ORDERPRODUCT."` AS OP
							LEFT JOIN `".WPSG_TBL_META."` AS M ON (M.`meta_table` = 'WPSG_TBL_ORDERPRODUCT' AND M.`target_id` = OP.`id` AND M.`meta_key` = 'ticket_id')
					WHERE
						OP.`p_id` = %d AND
						M.`meta_value` = %d
					GROUP BY
						OP.`id`
				", $this->getProductId(), $oTicket->getId()), ARRAY_A);
				
				foreach ($results as $row) {
				 
					$oOrderProduct = \wpsg_order_product::getInstance(intval($row['order_product_id']));
					
					if (intval($oOrderProduct->getMeta('ticket_id', false, 0)) === $oTicket->getId()) {
						
						for ($amount_index = 0; $amount_index < $oOrderProduct->getAmount(); $amount_index++) {
							
							$arReturn[] = TicketBooking::getInstanceByBookingKey(TicketBooking::encodeBookingKey($oOrderProduct->getId(), $amount_index));
							
						}
						
					}
					
				}
				
			}
			
			return $arReturn;
		
		}
		
		public function addTickets(string $from_date, string $from_time, string $to_date, string $to_time, int $slots): void {
		
			global $wpdb;
   
			$from_date = ($from_date !== "0000-00-00") ? $from_date : null;
			$to_date = ($to_date !== "0000-00-00") ? $to_date : null;
			$from_time = ($from_time !== "00:00") ? $from_time : null;
			$to_time = ($to_time !== "00:00") ? $to_time : null;
		
			$data = array(
				'p_id' => $this->getProductId(),
				'from_date' => $from_date,
				'from_time' => $from_time,
				'to_date' => $to_date,
				'to_time' => $to_time,
				'slots' => $slots,
				'state' => Ticket::STATE_ACTIVE
			);
		 
			$wpdb->insert($wpdb->prefix . "wpsg_tickets", $data);
		
		}
		
		/**
		 * @return Ticket[]
		 */
		public function getAvailableTickets(): array {
			
			/** @var Ticket[] $arTicket */
			$arTicket = Ticket::find([
				'product_id' => $this->getProductId(),
				'state' => Ticket::STATE_ACTIVE
			], null, true);
			
			foreach ($arTicket as $k => $oTicket) {
				
				if (!$oTicket->isAvailable()) unset($arTicket[$k]);
				
			}
			
			return $arTicket;
			
		}
		
		public static function isTicketProduct(int $product_id): bool {

			if ($product_id <= 0) return false;
			
			return \wpsg_product::getInstance($product_id)->getMeta('ticket_product', '0') === '1';
			
		}
		
		public static function getInstance($id) {
		
			if (!isset(self::$arInstances[$id])) self::$arInstances[$id] = new self($id);
		
			return self::$arInstances[$id];
		}
	
	}