<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 31.10.23
	 * @time: 13:22
	 */
	
	namespace wpsgTicket;
	
	use chillerlan\QRCode\Data\QRMatrix;
	use chillerlan\QRCode\Output\QROutputInterface;
	use chillerlan\QRCode\QRCode;
	use chillerlan\QRCode\QROptions;
	
	abstract class TicketRenderer {
		
		private static function SetTextColor(&$pdf, $strHEXCode) {
			
			if ($strHEXCode[0] == '#') $strHEXCode = substr($strHEXCode, 1);
		
		    if (strlen($strHEXCode) == 6) list($r, $g, $b) = array(
				$strHEXCode[0].$strHEXCode[1],
				$strHEXCode[2].$strHEXCode[3],
				$strHEXCode[4].$strHEXCode[5]
			);
		    else if (strlen($strHEXCode) == 3) list($r, $g, $b) = array($strHEXCode[0].$strHEXCode[0], $strHEXCode[1].$strHEXCode[1], $strHEXCode[2].$strHEXCode[2]);
		    else
		        return false;
		
		    $r = hexdec($r); $g = hexdec($g); $b = hexdec($b);
				  
			$pdf->setTextColor($r, $g, $b);
			
		}
		
		public static function render(?int $emplate_id = null, ?int $order_product_id = null, ?int $amount_index = null, bool $out = true): void {
		
			if ($order_product_id !== null && $amount_index !== null) {
				
				$oTicketBooking = TicketBooking::getInstanceByBookingKey(TicketBooking::encodeBookingKey($order_product_id, $amount_index));
				$ticket_booking_id = strval($oTicketBooking->getId());
				
				$ticket_file = $oTicketBooking->getTicketPDFPath();
				
				if (file_exists($ticket_file) && $out === true) {
					
					\wpsg_header::PDFPlugin($ticket_file); exit;
				
				} else if (file_exists($ticket_file)) return;
				
				if ($emplate_id === null) {
					
					$emplate_id = intval(\wpsg_order_product::getInstance($order_product_id)->getProduct()->getMeta('ticket_template'));
					
				}
				
			} else {
				
				$ticket_booking_id = '';
				$ticket_file = null;
				
			}
			
			require_once WPSG_PATH_LIB.'FPDF_1.81/fpdf.php';
			require_once WPSG_PATH_LIB.'FPDI_2.2.0/autoload.php';
			require_once WPSG_PATH_VIEW.'../mods/mod_ticket/lib/vendor/autoload.php';
			
			$oTickeTemplate = TicketTemplate::getInstance($emplate_id);
			$arConfig = $oTickeTemplate->getConfig();
			
			$w = wpsg_tf($arConfig['width']);
			$h = wpsg_tf($arConfig['height']);
			
			//$pdf = new \setasign\Fpdi\Fpdi((($w > $h)?'L':'P'), 'mm', [$w, $h]);
			$pdf = new PDF((($w > $h)?'L':'P'), 'mm', [$w, $h]);
			$pdf->SetAutoPageBreak(false);
			$pdf->AddPage();
			
			if ($oTickeTemplate->hasBackground()) {
				
				$bg_file = $oTickeTemplate->getBgFilePath();
				$info = pathinfo($bg_file);
				
				if ($info['extension'] === 'jpg') {
				
					$pdf->image($bg_file, 0, 0, $w, $h, 'jpg');
					
				} else if ($info['extension'] === 'pdf') {
				
					$pdf->setSourceFile($bg_file);
					$tplidx = $pdf->importPage(1, '/MediaBox');
					$pdf->useTemplate($tplidx, 0, 0, 210);
					
				}
				
			}
			
			foreach ($arConfig['arField'] as $c) {
				
				if ($c['text'] === '%qr%') {
					
					$pdf->SetAlpha(1);
					
					if ($order_product_id !== null) {
						
						$qr = Plugin::getInstance()->buildCode($order_product_id, $amount_index);
						
					} else {
						
						$qr = base64_encode('123456789');
						
					}
					
					
					$tmpfname = tempnam("/tmp", "wpsgTicket").'.png';
					
					$qrCodeOptions = new QROptions();
					$qrCodeOptions->quietzoneSize = 0;
					$qrCodeOptions->scale = 10;
					$qrCodeOptions->eccLevel = \chillerlan\QRCode\QRCode::ECC_Q;
					
					(new \chillerlan\QRCode\QRCode($qrCodeOptions))->render($qr, $tmpfname);
					
					$pdf->image($tmpfname, wpsg_tf($c['x']), wpsg_tf($c['y']), wpsg_tf($c['w']), wpsg_tf($c['h']));
					
				} else {
					
					if ($order_product_id !== null) {
						
						$oOrderProduct = \wpsg_order_product::getInstance($order_product_id);
						
						$c['text'] = \wpsg_ShopController::getShop()->replaceUniversalPlatzhalter(
							$c['text'],
							$oOrderProduct->getOrder()->getId(),
							$oOrderProduct->getOrder()->getCustomer()->getId(),
							null,
							$oOrderProduct->getProductId(),
							[
								'/%ticketbooking_id%/i' => $ticket_booking_id
							],
							$oOrderProduct->getProductId()
						);
						
					}
				 
					$font_file = WPSG_PATH_LIB.'FPDF_1.81/font/'.$c['font'];

					if (file_exists($font_file)) {
						
						global $name;
						
						include $font_file;
					
						$pdf->AddFont($name, '', $c['font']);
						$pdf->SetFont($name, '', $c['fontsize']);
						
						if (in_array($c['align'], ['C', 'R'])) {
							
							$text_width = $pdf->GetStringWidth($c['text']);
							
							if ($c['align'] === 'C') $c['x'] -= $text_width / 2;
							else if ($c['align'] === 'R') $c['x'] -= $text_width;
							
						}
						
					}
					
					self::SetTextColor($pdf, $c['color']);
					$pdf->setXY($c['x'], $c['y']);
					
					if (intval($c['opaque']) !== 100) {
						
						$pdf->SetAlpha(intval($c['opaque']) / 100);
						
					} else {
						
						$pdf->SetAlpha(1);
						
					}
					
					if ($c['angle'] !== '' && wpsg_tf($c['angle']) != 0) {
						
						$pdf->TextWithRotation($c['x'], $c['y'], $c['text'], wpsg_tf($c['angle']), 0);
						
					} else {
						
						$pdf->MultiCell(0, 3, $c['text']);
					
					}
				
				}
				
			}
			
			if ($ticket_file !== null) $pdf->output('F', $ticket_file);
			
			if ($out === true) $pdf->output();
			
		}
		
	}