<?php
	
	declare(strict_types=1);
	
	/**
	 * @author: Daniel Schmitzer (daniel@maennchen1.de)
	 * @date: 30.10.23
	 * @time: 16:00
	 */
	
	namespace wpsgTicket;
	
	class TicketTemplate {
		
		private static array $arInstances = [];
		
		private int $id;
		private array $data;
		private array $config;
		
		public function __construct(int $id) {
		
			global $wpdb;
			
			$this->id = $id;
			
			$query = $wpdb->prepare("SELECT * FROM `".WPSG_TBL_TICKET_TEMPLATES."` WHERE id = %d", $id);
			$this->data = $wpdb->get_row($query, ARRAY_A);
			
			$this->config = json_decode($this->data['config'], true);
		
		}
		
		public function getConfig(): array { return $this->config; }
		
		public function getStoragePath(): string {
			
			$shop = \wpsg_ShopController::getShop();
			
			if ($shop->isMultiBlog()) $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_ticket/templates/';
			else $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_ticket/templates/';
   
			$shop->protectDirectory($path.intval($this->id).DIRECTORY_SEPARATOR);
			
			return $path.intval($this->id).DIRECTORY_SEPARATOR;
			
		}
		
		public function toJSON(): array {
			
			$r = [
				'id' => $this->id,
				'template_name' => $this->getLabel(),
				'config' => $this->config,
				'url_preview' => \add_query_arg([
					'action' => 'wpsg_ticket',
					'ticket_action' => 'preview',
					'template_id' => $this->id
				], \admin_url('admin-ajax.php'))
			];
			
			if ($this->hasBackground()) {
				
				$r['hasBackground'] = true;
				$r['url_bg_download'] = \add_query_arg([
					'action' => 'wpsg_ticket',
					'ticket_action' => 'getBackground',
					'template_id' => $this->id
				], \admin_url('admin-ajax.php'));
				
			} else {
				
				$r['hasBackground'] = false;
				
			}
			
			return $r;
			
		}
		
		public function getId(): int {
			
			return $this->id;
			
		}
		
		public function getBgFilePath(): ?string {
			
			$path = $this->getStoragePath();
			
			foreach(scandir($path) as $f) {
				
				if (!in_array($f, ['.', '..', '.htaccess'])) {
					
					return $path.$f;
					
				}
				
			}
			
			return null;
			
		}
		
		public function hasBackground(): bool {
		
			return $this->getBgFilePath() !== null;
		
		}
		
		public function getLabel(): string { return $this->data['template_name']; }
		
		/**
		 * @param array $arFilter
		 * @param array|null $arPager
		 * @param bool $getObject
		 * @return array|TicketTemplate[]
		 */
		public static function find(array $arFilter = [], ?array $arPager = null, bool $getObject = false): array {
		
			global $wpdb;
			
			$strQueryWHERE = self::getQueryWHERE($arFilter);
			
			if ($arPager !== null) {
				
				$per_page = intval($arPager['per_page']??1);
				$page = intval($arPager['page']??1);
				
				$offset = ($page - 1) * $per_page;
				
				$arID = $wpdb->get_results("
					SELECT
						O.`id`
					FROM
						`".WPSG_TBL_TICKET_TEMPLATES."` AS O
					WHERE
						1
						".$strQueryWHERE."
					LIMIT
						".intval($per_page)." OFFSET ".intval($offset)."
					", ARRAY_A);
				
				$r = []; foreach ($arID as $id) {
					
					if ($getObject) $r[] = TicketTemplate::getInstance(intval($id['id']));
					else $r[] = TicketTemplate::getInstance(intval($id['id']))->toJSON();
					
				}
				
				return $r;
				
			} else {
				
				$arID = $wpdb->get_results("
					SELECT
						O.`id`
					FROM
						`".WPSG_TBL_TICKET_TEMPLATES."` AS O
					WHERE
						1
						".$strQueryWHERE, ARRAY_A);
				
				$r = []; foreach ($arID as $id) {
					
					if ($getObject) $r[] = TicketTemplate::getInstance(intval($id['id']));
					else $r[] = TicketTemplate::getInstance(intval($id['id']))->toJSON();
					
				}
				
				return $r;
				
			}
		
		}
		
		public static function getQueryWHERE(array $arFilter): string {
			
			global $wpdb;
			
			$strQueryWHERE = " AND `deleted` != '1' ";
			
			return $strQueryWHERE;
			
		}
		
		public static function getInstance($id): TicketTemplate {
   
			if (!isset(self::$arInstances[$id])) self::$arInstances[$id] = new self($id);
			
        	return self::$arInstances[$id];
    
		}
		
		public static function save(array $template_data): void {
			
			global $wpdb;
			
			if ($template_data['id'] <= 0) {
				
				$wpdb->insert(WPSG_TBL_TICKET_TEMPLATES, [
					'config' => json_encode($template_data['config']),
					'template_name' => $template_data['template_name']
				], [
					'%s',
					'%s'
				]);
				
			} else {
				
				$wpdb->update(WPSG_TBL_TICKET_TEMPLATES, [
					'config' => json_encode($template_data['config']),
					'template_name' => $template_data['template_name']
				], [
					'id' => $template_data['id']
				]);
				
			}
			
		}
		
	}